#include <iostream>
#include <string>
using namespace std;
#include "BMPImage.h"
#include <GL/glut.h>
#include "math.h"
#include "texture.h"
#include <algorithm>

GLsizei winWidth = 960, winHeight = 480;
BMPImage I1, I2;
/* I1 and I2 will hold our two main images.  */
GLuint texID[2];
#define MAXLINES 20
float lines1[MAXLINES][2][2];
int numLines1 = -1;
float lines2[MAXLINES][2][2];
int numLines2 = -1;

void setSubwindow(int w) {
	if (w == 0) {
		glViewport(0, 0, winWidth/2, winHeight);
	}
	if (w == 1) {
		glViewport(winWidth/2, 0, winWidth/2, winHeight);
	}

}

void displayTexture (void) {
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex2f(0, 0);
		glTexCoord2f(1,0);
		glVertex2f(1, 0);
		glTexCoord2f(1,1);
		glVertex2f(1, 1);
		glTexCoord2f(0,1);
		glVertex2f(0,1);
	glEnd();
	glDisable(GL_TEXTURE_2D);
}

void displayLines(float lines[MAXLINES][2][2], int numLines) {
	glLineWidth(5.0);
	glColor3f(1.0, 0.0, 0.0);
	glBegin(GL_LINES);
	for (int i = 0; i <= numLines; ++i) {
		glVertex2f(lines[i][0][0], lines[i][0][1]);
		glVertex2f(lines[i][1][0], lines[i][1][1]);
	}
	glEnd();
}

void displayFcn (void)
{
	glClear (GL_COLOR_BUFFER_BIT);
	glColor3f (1,0,0);
	glMatrixMode(GL_MODELVIEW);

	setSubwindow(0);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texID[0]);
	displayTexture();
    displayLines(lines1, numLines1);

	setSubwindow(1);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texID[1]);
	displayTexture();
	displayLines(lines2, numLines2);

	glutSwapBuffers ( ); 
}

void init(void)
{
	glClearColor (0.0, 0.0, 1.0, 0.0);

	glMatrixMode (GL_PROJECTION);
	gluOrtho2D (0.0, 1, 0.0, 1);
	glMatrixMode (GL_MODELVIEW);	

	char fname1[80];
	char fname2[80];

	cout << endl << "Enter filename image 1:";
	cin>>fname1;
	cout << endl << "Enter filename image 2:";
	cin>>fname2;
	

	int status1 = I1.load(fname1);
	int status2 = I2.load(fname2);

	imageTexture(I1, texID);
	imageTexture(I2, texID+1);
}

void timer(int id){
	glutTimerFunc(30, timer, 1);
	glutPostRedisplay();
}

void coordWindow(int x, int y, int& w, float& xw, float& yw) {
	float xf = x;
	float yf = y;
	if (x < winWidth/2) {
		w = 0;
	}
	else {
		w = 1;
		xf = x - winWidth/2;
	}
	xw = 2*xf/winWidth;
	yw = 1 - yf/winHeight;
}

void mouseButtons(int button, int state, int x, int y) {
	float xw, yw;		
	int w;
	if ((button == GLUT_LEFT_BUTTON) && (state == GLUT_DOWN) ) {
		coordWindow(x,y, w, xw, yw);
		if (w == 0) {
			lines1[numLines1+1][0][0] = xw;
			lines1[numLines1+1][0][1] = yw;
			lines1[numLines1+1][1][0] = xw;
			lines1[numLines1+1][1][1] = yw;
			numLines1 += 1;
		}
		else {
			lines2[numLines2+1][0][0] = xw;
			lines2[numLines2+1][0][1] = yw;
			lines2[numLines2+1][1][0] = xw;
			lines2[numLines2+1][1][1] = yw;
			numLines2 += 1;
		}
	}
	else if ((button == GLUT_LEFT_BUTTON) && (state == GLUT_UP)) {
		coordWindow(x,y, w, xw, yw);
		if (w == 0) {
			lines1[numLines1+1][1][0] = xw;
			lines1[numLines1+1][1][1] = yw;
		}
		else {
			lines2[numLines2+1][1][0] = xw;
			lines2[numLines2+1][1][1] = yw;
			}	
	}
	glutPostRedisplay();
}

void mouseDrag(int x, int y) {
	float xw, yw;		
	int w;
	coordWindow(x,y, w, xw, yw);
	if (w == 0) {
		lines1[numLines1][1][0] = xw;
		lines1[numLines1][1][1] = yw;
	}
	else {
		lines2[numLines2][1][0] = xw;
		lines2[numLines2][1][1] = yw;
	}
}

void keyboard(unsigned char key, int x, int y) {
	if (key == 'u') {
		numLines1 -= 1;
	}
	if (key == 'v') {
		numLines2 -= 1;
	}
	glutPostRedisplay();
}

void main (int argc, char** argv)
{
	glutInit (&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
	glutInitWindowPosition (50, 100);
	glutInitWindowSize (winWidth, winHeight);

	glutCreateWindow ("Interactive Morphing");

	init();
	glutDisplayFunc (displayFcn);
	glutMouseFunc (mouseButtons);
	glutMotionFunc (mouseDrag); 
	glutKeyboardFunc (keyboard);
	glutTimerFunc(30, timer, 1);

	glutMainLoop ( );
}